import { __decorate } from "tslib/tslib.es6.js";
/**
@license
Copyright 2018 Google Inc. All Rights Reserved.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

import { addHasRemoveClass, BaseElement } from "../mwc-base/base-element.js";
import MDCFadingTabIndicatorFoundation from "../tab-indicator/fading-foundation.js";
import MDCSlidingTabIndicatorFoundation from "../tab-indicator/sliding-foundation.js";
import { html, property, query } from "lit-element/lit-element.js";
import { classMap } from "lit-html/directives/class-map.js";
export class TabIndicatorBase extends BaseElement {
  constructor() {
    super(...arguments);
    this.icon = '';
    this.fade = false;
  }

  get mdcFoundationClass() {
    return this.fade ? MDCFadingTabIndicatorFoundation : MDCSlidingTabIndicatorFoundation;
  }

  render() {
    const contentClasses = {
      'mdc-tab-indicator__content--icon': this.icon,
      'material-icons': this.icon,
      'mdc-tab-indicator__content--underline': !this.icon
    };
    return html`
      <span class="mdc-tab-indicator ${classMap({
      'mdc-tab-indicator--fade': this.fade
    })}">
        <span class="mdc-tab-indicator__content ${classMap(contentClasses)}">${this.icon}</span>
      </span>
      `;
  }

  updated(changedProperties) {
    if (changedProperties.has('fade')) {
      this.createFoundation();
    }
  }

  createAdapter() {
    return Object.assign(Object.assign({}, addHasRemoveClass(this.mdcRoot)), {
      computeContentClientRect: () => this.contentElement.getBoundingClientRect(),
      setContentStyleProperty: (prop, value) => this.contentElement.style.setProperty(prop, value)
    });
  }

  computeContentClientRect() {
    return this.mdcFoundation.computeContentClientRect();
  }

  activate(previousIndicatorClientRect) {
    this.mdcFoundation.activate(previousIndicatorClientRect);
  }

  deactivate() {
    this.mdcFoundation.deactivate();
  }

}

__decorate([query('.mdc-tab-indicator')], TabIndicatorBase.prototype, "mdcRoot", void 0);

__decorate([query('.mdc-tab-indicator__content')], TabIndicatorBase.prototype, "contentElement", void 0);

__decorate([property()], TabIndicatorBase.prototype, "icon", void 0);

__decorate([property({
  type: Boolean
})], TabIndicatorBase.prototype, "fade", void 0);